"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
import datetime
from alerts import Alerts
from helpers.info_dict import InfoDict
from constants import NODE_DOWN_RETRY_TIME, NODE_VISIBILITY_RETRY_TIME


class NodeStats(Alerts):

    def __init__(self, cluster_id, node_address, node_status):

        Alerts.__init__(self, cluster_id)
        self._stats = InfoDict()
        self._node_address = node_address
        self.attr_to_handler = (
            ('free-pct-disk', 'on_free_pct_disk_change'),
            ('client_connections', 'on_client_connections_change'),
            ('cluster_visibilty', 'on_cluster_visibility_change'),
            ('queue', 'on_queue_change')
        )

        status = "green" if node_status == "on" else "red"
        self.free_pct_disk_data = dict(status="green", last_id=None)
        self.queue_alert_data = dict(status="green", last_id=None)
        self.client_connections_alert_data = dict(status="green", last_id=None)
        self.node_status_alert_data = dict(status=status, last_id=None, last_node_down_timestamp=None)
        self.cluster_visibility_alert_data = dict(status="green", last_id=None, last_cluster_invisible_timestamp=None)

    def on_free_pct_disk_change(self):

        messages = dict(
            red="Free disk space on node <strong>%s</strong> below 5%%",
            yellow="Free disk space on node <strong>%s</strong> below 10%%",
            green="Free disk space on node <strong>%s</strong> above 10%% now"
        )

        free_pct_disk = self._stats.get_as_int('free-pct-disk')
        total_disk = self._stats.get_as_int('total-bytes-disk')
        if total_disk is None or free_pct_disk is None or total_disk <= 0:
            return

        alert_status = self.get_alert_status(
            free_pct_disk.is_less_than(5),  # Red alert condition
            free_pct_disk.is_less_than(10)  # Yellow alert condition
        )

        if alert_status != self.free_pct_disk_data['status']:
            self.free_pct_disk_data['status'] = alert_status
            self.free_pct_disk_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._node_address)
                , alert_status, self._node_address, "alert", self.free_pct_disk_data['last_id'])

    def on_client_connections_change(self):

        messages = dict(
            red="Client connections to node <strong>%s</strong> above 95%% of limit",
            yellow="Client connections to node <strong>%s</strong> above 90%% of limit",
            green="Client connections to node <strong>%s</strong> below 90%% of limit now"
        )

        client_connections = self._stats.get_as_int('client_connections')
        proto_fd_max = self._info.get_as_int('proto-fd-max')

        if client_connections is None or proto_fd_max is None:
            return

        alert_status = self.get_alert_status(
            client_connections.percentage(proto_fd_max).is_greater_than(95),  # Red alert condition
            client_connections.percentage(proto_fd_max).is_greater_than(90)   # Yellow alert condition
        )

        if alert_status != self.client_connections_alert_data['status']:
            self.client_connections_alert_data['status'] = alert_status
            self.client_connections_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._node_address),
                alert_status, self._node_address, "alert", self.client_connections_alert_data['last_id'])

    def on_status_change(self):

        messages = dict(
            red="Node <strong>%s</strong> is down",
            green="Node <strong>%s</strong> is up now"
        )

        node_status = self._stats.get('node_status')

        alert_status = self.get_alert_status(
            node_status == "off",  # Red alert condition
            None                   # Yellow alert condition
        )

        if alert_status != self.node_status_alert_data['status']:
            if alert_status == "red":
                time_now = datetime.datetime.now()
                if self.node_status_alert_data['last_node_down_timestamp'] is None:
                    self.node_status_alert_data['last_node_down_timestamp'] = time_now
                    return
                elif (time_now - self.node_status_alert_data['last_node_down_timestamp']).seconds < NODE_DOWN_RETRY_TIME:
                    return

            self.node_status_alert_data['last_node_down_timestamp'] = None
            self.node_status_alert_data['status'] = alert_status
            self.node_status_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._node_address)
                , alert_status, self._node_address, "alert", self.node_status_alert_data['last_id'])

    def on_cluster_visibility_change(self):

        messages = dict(
            red="Node <strong>%s</strong> is not visible to rest of the cluster",
            green="Node <strong>%s</strong> is now visible to rest of the cluster"
        )

        cluster_visibility = self._stats.get('cluster_visibility')

        alert_status = self.get_alert_status(
            cluster_visibility is False,  # Red alert condition
            None                          # Yellow alert condition
        )

        if alert_status != self.cluster_visibility_alert_data['status']:
            if alert_status == "red":
                time_now = datetime.datetime.now()
                if self.cluster_visibility_alert_data['last_cluster_invisible_timestamp'] == None:
                    self.cluster_visibility_alert_data['last_cluster_invisible_timestamp'] = time_now
                    return
                elif (time_now - self.cluster_visibility_alert_data['last_cluster_invisible_timestamp']).seconds < NODE_VISIBILITY_RETRY_TIME:
                    return

            self.cluster_visibility_alert_data['last_cluster_invisible_timestamp'] = None
            self.cluster_visibility_alert_data['status'] = alert_status
            self.cluster_visibility_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._node_address),
                alert_status, self._node_address, "alert", self.cluster_visibility_alert_data['last_id'])

    def on_queue_change(self):

        queue_limit = 10000
        messages = dict(
            yellow="Transactions pending in queue for node <strong>%s</strong> is greater than %s",
            green="Transactions pending in queue for node <strong>%s</strong> is less than %s now"
        )

        queue = self._stats.get_as_int('queue')
        if queue is None:
            return

        alert_status = self.get_alert_status(
            None,                                # Red alert condition
            queue.is_greater_than(queue_limit),  # Yellow alert condition
        )
        if alert_status != self.queue_alert_data['status']:
            self.queue_alert_data['status'] = alert_status
            self.queue_alert_data['last_id'] = self.create_alert(
                messages[alert_status] % (self._node_address, queue_limit)
                , alert_status, self._node_address, "alert", self.queue_alert_data['last_id'])

    def generate_alerts(self, stats, info=None):

        if not stats:
            return
        self._stats = stats

        self.on_status_change()
        Alerts.generate_alerts(self, stats, info)
