"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
import sys
import functools
import datetime
import threading
import traceback
from libs import base
from flask import json, current_app
from flask_login import session, current_user
from models import cluster
from models.user import User
from constants import *
from helpers.synchronizing import synchronous
from werkzeug.exceptions import Unauthorized

class ClusterManager(object):
    """It manages the all current cluster objects"""

    def __init__(self):

        self._clusters = {}
        self.clusters_lock = threading.RLock()

    @synchronous('clusters_lock')
    def create_cluster(self, node_address, user=None, password=None):

        cluster_info = self.get_cluster_by_node(node_address)
        if cluster_info is None:
            ip, port = node_address.split(":")
            return cluster.Cluster(ip, port, user=user, password=password)
        return EXISTING_CLUSTER

    def add_cluster(self, cluster):

        self._clusters[cluster.get_cluster_id()] = cluster

    def remove_cluster(self, cluster_id):

        del(self._clusters[cluster_id])

    def get_cluster_by_id(self, cluster_id):

        return self._clusters.get(cluster_id, None)

    def get_cluster_by_node(self, node_address):

        for cluster_id, cluster_obj in list(self._clusters.items()):
            if node_address in cluster_obj.get_nodes():
                return dict(cluster_id=cluster_id, nodes=cluster_obj.get_node_list())
        return None

    def get_cluster_obj_by_node(self, node_address):

        for cluster_id, cluster_obj in list(self._clusters.items()):
            if node_address in cluster_obj.get_nodes():
                return cluster_obj
        return None

    def get_cluster_info(self, host_address, user=None, password=None):

        update_interval = current_app.config.get("UPDATE_INTERVAL")
        try:
            ip, port = host_address.split(":")
        except:
            return dict(error="Unable to connect - %s" % host_address)

        if user and password and base.check_authenticity_of_user(ip, port,
                                               user, password) in [UNRECOGNISED_USER, BAD_PASSWORD, BAD_CREDENTIAL]:
            raise Unauthorized("Unauthenticated user!")

        is_security_enabled = base.is_security_enabled(ip, port)

        if is_security_enabled == UNABLE_TO_CONNECT:
            return dict(error="Unable to connect - %s" % host_address)

        if not is_security_enabled:
            node_address = base.get_self_address(ip, port)
        if is_security_enabled and user is not None:
            node_address = base.get_self_address(ip, port, user=user, password=password)

        if isinstance(node_address, int) and node_address == UNABLE_TO_CONNECT:
            return dict(error="Unable to connect - %s" % host_address)

        if type(node_address) == int and node_address != UNABLE_TO_CONNECT:
            return dict(security_enabled=is_security_enabled, cluster_id=None)

        cluster_info = self.get_cluster_by_node(node_address)

        if cluster_info is not None:
            cluster_info.update(update_interval=update_interval)
            cluster_info.update(security_enabled=is_security_enabled)
            # TODO: Add current user to cluster object's users' dict
            # DONE
            if is_security_enabled:
                user_obj = User(
                    user, password, host=host_address
                    )
                cluster_obj = self.get_cluster_obj_by_node(node_address)
                cluster_obj.add_user(user_obj)
            return cluster_info

        if not is_security_enabled:
            cluster = self.create_cluster(node_address)
        if is_security_enabled and user is not None:
            cluster = self.create_cluster(node_address, user=user, password=password)

        if cluster == UNABLE_TO_CONNECT or cluster == ERROR_OCCURRED:
            return dict(error="Unable to connect - %s" % host_address)
        elif cluster == EXISTING_CLUSTER:
            cluster_info = self.get_cluster_by_node(node_address)
            if cluster_info is not None:
                cluster_info.update(update_interval=update_interval)
                cluster_info.update(security_enabled=is_security_enabled)
                # TODO: Add current user to cluster object's users' dict
                # DONE
                if is_security_enabled:
                    user_obj = User(
                        user, password, host=host_address
                        )
                    cluster_obj = self.get_cluster_obj_by_node(node_address)
                    cluster_obj.add_user(user_obj)
                return cluster_info
        else:
            self.add_cluster(cluster)
            return dict(cluster_id=cluster.get_cluster_id(),
                        nodes=cluster.get_node_list(),
                        update_interval=update_interval,
                        security_enabled=is_security_enabled
                        )

    def get_clusters(self):

        return self._clusters

    def remove_unused_clusters(self):

        current_time = datetime.datetime.now()

        for cluster_id, cluster_obj in list(self._clusters.items()):
            elapsed_time = current_time - cluster_obj.get_last_access_time()
            if elapsed_time.seconds > CLUSTER_TIMEOUT * 60:
                self.remove_cluster(cluster_id)

    def check_cluster_validity(self):

        def decorator(fn):

            @functools.wraps(fn)
            def f(*args, **kwargs):

                #if str(kwargs['cluster']) != session.get('cluster_id', ''):
                    #return json.dumps(dict(error="Invalid cluster id : %s" % kwargs["cluster"]))

                cluster_id = session.get('cluster_id', None) or kwargs.get('cluster', None)

                cluster = self.get_cluster_by_id(cluster_id)
                if cluster is None:
                    return json.dumps(dict(error="Invalid cluster id : %s" % kwargs["cluster"]))

                if cluster.is_secured() and current_user.get_id() is None:
                    return current_app.login_manager.unauthorized()

                cluster.update_last_access_time()
                kwargs["cluster"] = cluster
                try:
                    return fn(*args, **kwargs)
                except Unauthorized as ua:
                    return ua
                except Exception:
                    sys.stderr.write("args:%s\nkwargs:%s" % (args, kwargs))
                    sys.stderr.write((traceback.format_exc()))
                    raise Exception("Internal server error!")

            return f
        return decorator
